# ==========================================================================
#
# ZoneMinder Foscam FI8908W IP Control Protocol Module, $Date: 2009-11-25 09:20:00 +0000 (Wed, 04 Nov 2009) $, $Revision: 0001 $
# Copyright (C) 2001-2008 Philip Coombes
# Modified for use with Foscam FI8908W IP Camera by Dave Harris
#
# ==========================================================================
#
# This module contains the implementation of the Foscam FI8908W IP camera control
# protocol
#
package ZoneMinder::Control::FoscamFI8908W;

use 5.006;
use strict;
use warnings;

require ZoneMinder::Base;
require ZoneMinder::Control;

our @ISA = qw(ZoneMinder::Control);

our $VERSION = $ZoneMinder::Base::VERSION;

# ==========================================================================
#
# Foscam FI8908W IP Control Protocol
#
# ==========================================================================

use ZoneMinder::Debug qw(:all);
use ZoneMinder::Config qw(:all);

 use Time::HiRes qw( usleep );
 
 sub new
 { 
 
 my $class = shift;
 my $id = shift;
 my $self = ZoneMinder::Control->new( $id );
 my $logindetails = "";
 bless( $self, $class );
 srand( time() );
 return $self;
 }
 
 our $AUTOLOAD;
 
 sub AUTOLOAD
 {
 my $self = shift;
 my $class = ref($self) || croak( "$self not object" );
 my $name = $AUTOLOAD;
 $name =~ s/.*://;

Info( "Autoload $name" );
Debug( "Autoload $name" );
 if ( exists($self->{$name}) )
 {
 return( $self->{$name} );
 }
 Fatal( "Can't access $name member of object of class $class" );
 }
 our $stop_command;
 
 sub open
 {
 my $self = shift;
 
 $self->loadMonitor();
 
 use LWP::UserAgent;
 $self->{ua} = LWP::UserAgent->new;
 $self->{ua}->agent( "ZoneMinder Control Agent/".ZM_VERSION );
 
 $self->{state} = 'open';
 }
 
 sub close
 { 
 my $self = shift;
 $self->{state} = 'closed';
 }
 
 sub printMsg
 {
 my $self = shift;
 my $msg = shift;
 my $msg_len = length($msg);
 
 Debug( $msg."[".$msg_len."]" );
 }
 
 sub sendCmd
 {
 my $self = shift;
 my $cmd = shift;
 my $result = undef;
 printMsg( $cmd, "Tx" );
 
 my $req = HTTP::Request->new( GET=>"http://".$self->{Monitor}->{ControlAddress}."/$cmd"."&user=admin&pwd=51095109" );
 my $res = $self->{ua}->request($req);
 
 if ( $res->is_success )
 {
 $result = !undef;
 }
 else
 {
 Error( "Error check failed:'".$res->status_line()."'" );
 }
 
 return( $result );
 }
 
 sub reset
 {
 my $self = shift;
 Debug( "Camera Reset" );
 my $cmd = "reboot.cgi";
 $self->sendCmd( $cmd );
 }
 
 #Up Arrow
 sub moveConUp
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Up" );
 my $cmd = "decoder_control.cgi?command=0&onestep=1";
 $self->sendCmd( $cmd );
 }
 
 #Down Arrow
 sub moveConDown
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Down" );
 my $cmd = "decoder_control.cgi?command=2&onestep=1";
 $self->sendCmd( $cmd );
 }
 
 #Left Arrow
 sub moveConLeft
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Left" );
# my $cmd = "decoder_control.cgi?command=4";
 my $cmd = "decoder_control.cgi?command=4&onestep=1";
 $self->sendCmd( $cmd );
 }
 
 #Right Arrow
 sub moveConRight
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Right" );
# my $cmd = "decoder_control.cgi?command=6";
 my $cmd = "decoder_control.cgi?command=6&onestep=1";
 $self->sendCmd( $cmd );
 }
 
 #Diagonally Up Right Arrow
 sub moveConUpRight
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Diagonally Up Right" );
 my $cmd = "decoder_control.cgi?command=91&onestep=1";
 $self->sendCmd( $cmd );
 }
 
 #Diagonally Down Right Arrow
 sub moveConDownRight
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Diagonally Down Right" );
 my $cmd = "decoder_control.cgi?command=93&onestep=1";
 $self->sendCmd( $cmd );
 }
 
 #Diagonally Up Left Arrow
 sub moveConUpLeft
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Diagonally Up Left" );
 my $cmd = "decoder_control.cgi?command=90&onestep=1";
 $self->sendCmd( $cmd );
 }
 
 #Diagonally Down Left Arrow
 sub moveConDownLeft
 {
 my $self = shift;
 $stop_command = "1";
 Debug( "Move Diagonally Down Left" );
 my $cmd = "decoder_control.cgi?command=92&onestep=1";
 $self->sendCmd( $cmd );
 }

 #Up Rel
 sub moveRelUp
 {
 my $self = shift;
 my $params = shift;
 my $step = $self->getParam( $params, 'panstep' );
 $stop_command = "1";
 Debug( "Move Rel Up" );
 my $cmd = "decoder_control.cgi?command=0&onestep=$step";
 $self->sendCmd( $cmd );
 }
 
 #Down Rel 
 sub moveRelDown
 {
 my $self = shift;
 my $params = shift;
 my $step = $self->getParam( $params, 'panstep' );
 $stop_command = "1";
 Debug( "Move Rel Down" );
 my $cmd = "decoder_control.cgi?command=2&onestep=$step";
 $self->sendCmd( $cmd );
 }


 #Left Rel 
 sub moveRelLeft
 {
 my $self = shift;
 my $params = shift;
 my $step = $self->getParam( $params, 'panstep' );
 Debug( "Step Left $step" );
 my $cmd = "decoder_control.cgi?command=4&onestep=$step";
 $self->sendCmd( $cmd );
 }

#Right Rel
 sub moveRelRight
 {
 my $self = shift;
 my $params = shift;
 my $step = $self->getParam( $params, 'panstep' );
 Debug( "Step Right $step" );
 my $cmd = "decoder_control.cgi?command=6&onestep=$step";
 $self->sendCmd( $cmd );
 }

 
 #Stop
 sub moveStop
 {
 my $self = shift;
 Debug( "Move Stop" );
 my $cmd = "decoder_control.cgi?command=1";
 $self->sendCmd( $cmd );
 }
 
 #Move Camera to Home Position
 sub presetHome
 {
 my $self = shift;
 Debug( "Home Preset" );
 my $cmd = "decoder_control.cgi?command=25";
 $self->sendCmd( $cmd );
 }

#Move Camera to Preset Position
sub presetGoto
{
    my $self = shift;
    my $params = shift;
    my $preset = $self->getParam( $params, 'preset' );
    Debug( "Goto Preset $preset" );
    #$preset - number of preset
    #go 1 - 31
    #go 2 - 33
    #go 3 - 35
    Info("Preset $preset");
    $preset=$preset*2+29;
    my $cmd = "decoder_control.cgi?command=$preset";
    $self->sendCmd( $cmd );
}

#Move Camera to PseudoMap
sub movePseudoMap
{
    my $self = shift;
    my $params = shift;
    my $xcoord = $self->getParam( $params, 'xcoord' );
    my $ycoord = $self->getParam( $params, 'ycoord' );
    my $direct=1; #Stop default
#   For FosCam 8908 hiight and width contain 14 "onestep" steps
    my $steps=14;
    my $x_steps=0;
    my $istep=0;
    Debug( "Move Map to $xcoord,$ycoord" );
    Info( "1 Move Map to $xcoord,$ycoord" );
#    Info("2 monitor Width".$self->{Monitor}->{Width});
#    Info("3 monitor Height".$self->{Monitor}->{Height});
#    6>  4<  2v 0^;
   if ( $xcoord < ( $self->{Monitor}->{Width})/2 ) 
   { # Left rotate
     Info("5 X dir Left");
     $direct=4;
   }
   if ( $xcoord > ( $self->{Monitor}->{Width})/2 ) 
   { #Right rotate
     Info("6 X dir Right");
     $direct=6;
   }
   $x_steps = abs(($self->{Monitor}->{Width})/2-$xcoord)/(($self->{Monitor}->{Width})/$steps) ;
#   $x_steps = round($x_steps);
    Info("x_steps= $x_steps");

#   if ( $ycoord < ( $self->{Monitor}->{Height})/2 ) 
#   {
#     Info("7 Y dir UP");
#     If ( $direct=1 )
#        {
#        #UP
#        $direct=0;
#        }
#     If ( $direct=6 )
#        {
#        #Right Up
#        $direct=91;
#        }
#         If ( $direct=4 )
#        {
#        #Left Up
#        $direct=90;
#        }
#   }
#   if ( $ycoord > ( $self->{Monitor}->{Height})/2 ) 
#   {
#     Info("8 Y dir Down");
#     If ( $direct=1 )
#        {
#        #Down
#        $direct=2;
#        }
#     If ( $direct=6 )
#        { #Right Down
#        Info("11 dir Right Down");
#        $direct=93;
#        }
#         If ( $direct=4 )
#        { #Left Down
#        Info("11 dir Left Down");
#        $direct=92;
#        }
#   }
   
 my $cmd = "decoder_control.cgi?command=$direct&onestep=1";
for($istep=1; $istep<=$x_steps; $istep++)
{
 $self->sendCmd( $cmd );
     Debug("Pseudomap FosCam command: $cmd ");
}
# $self->sendCmd( $cmd );
#     Info("96 CMD: $cmd ");
}



 
 1;